package models

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestAddOrUpdateAuthor(t *testing.T) {
	// Create test database
	assert.NoError(t, PrepareTestDatabase())

	// Bootstrap our test author
	testauthor := Author{Forename: "test", Lastname: "tsting"}

	// Create a new author
	author1, err := AddOrUpdateAuthor(testauthor)
	assert.NoError(t, err)

	// And anotherone
	_, err = AddOrUpdateAuthor(testauthor)
	assert.NoError(t, err)

	// As of now, we should have 2 author in total. Get the list and check.
	allauthors, err := ListAuthors("")
	assert.NoError(t, err)

	for _, author := range allauthors {
		assert.Equal(t, testauthor.Forename, author.Forename)
		assert.Equal(t, testauthor.Lastname, author.Lastname)
	}

	// Get the new author
	gotauthor, exists, err := GetAuthorByID(author1.ID)
	assert.NoError(t, err)
	assert.True(t, exists)
	assert.Equal(t, gotauthor.Forename, testauthor.Forename)

	// Pass an empty author to see if it fails
	_, err = AddOrUpdateAuthor(Author{})
	assert.Error(t, err)

	// Update the author
	testauthor.ID = 1
	testauthor.Forename = "Lorem Ipsum"
	author1updated, err := AddOrUpdateAuthor(testauthor)
	assert.NoError(t, err)
	assert.Equal(t, testauthor.Forename, author1updated.Forename)

	// Delete the author
	err = DeleteAuthorByID(author1.ID)
	assert.NoError(t, err)

	// Check if it is gone
	_, exists, err = GetAuthorByID(author1.ID)
	assert.NoError(t, err)
	assert.False(t, exists)
}
